import pygame
import sys
import pandas as pd

pygame.init()
screen  = pygame.display.set_mode((1200, 800))
pygame.display.set_caption('Muller Lyer')
screen.fill((255,255,255))
pygame.display.flip()

xL = 600 - 350
xR = 600 + 350
L = 80

class Button:
    def __init__(self):
        self.screen_rect = screen.get_rect()
        self.width, self.height = 200, 70
        self.button_color = (0,0,255)
        self.text_color = (255,255,255)
        self.font = pygame.font.SysFont(None, 58)
        self.rect = pygame.Rect(600-(self.width//2), 800 - (self.height * 2),
                                self.width, self.height)
        self.msg_image = self.font.render('OK', True, self.text_color,
                                          self.button_color)
        self.msg_image_rect = self.msg_image.get_rect()
        self.msg_image_rect.center = self.rect.center

    def draw_button(self):
        screen.fill(self.button_color, self.rect) 
        screen.blit(self.msg_image, self.msg_image_rect) 


def disp_pair(dL):
    screen.fill((255,255,255))
    #
    #          左右の矢軸の描画
    #
    pygame.draw.line(screen, (0,0,0), (xL,400), (xR+dL,400), width = 5)
    #
    #          左の矢羽根の描画
    #
    pygame.draw.line(screen, (0,0,0), (xL+L,400-L), (xL,400), width = 5)
    pygame.draw.line(screen, (0,0,0), (xL+L,400+L), (xL, 400), width = 5)
    #
    #          中央の矢羽根の描画
    #
    pygame.draw.line(screen, (0,0,0), (600-L, 400+L), (600, 400), width = 5)
    pygame.draw.line(screen, (0,0,0), (600-L, 400-L), (600, 400), width = 5)
    #
    #          右の矢羽根の描画
    #
    pygame.draw.line(screen, (0,0,0), (xR+L+dL, 400-L), (xR+dL, 400), width = 5)
    pygame.draw.line(screen, (0,0,0), (xR+L+dL, 400+L), (xR+dL, 400), width = 5)
    #
    #           ボタンの設置
    #
    global ok_button
    ok_button = Button()
    ok_button.draw_button()
    #
    #           ディスプレイ画面への表示
    #
    pygame.display.flip()
    pygame.time.wait(50)        #   50msecのブレーキ


def disp_longer():
    global dL
    if xR + L + dL + 1 < 1200:
        dL += 1                #   右の矢軸の伸長
    disp_pair(dL)

def disp_shorter():
    global dL
    if xR + dL - 1 > 600:
        dL -= 1                 #   右の矢軸の収縮
    disp_pair(dL)


n_trials = 20    #   総試行数
trials = []      #   試行数
conds = []       #   提示条件
resps = []       #   反応

for i_trial in range(n_trials):
    trials.append(i_trial+1)
    if i_trial % 2 == 1:
        dL = 100             #      最初に十分長い矢軸を提示する
        conds.append('U')
    else:
        dL = -200            #      最初に十分短い矢軸を提示する
        conds.append('L')
    disp_pair(dL)

    ck_go_L = False        #   矢軸を延ばすかどうかのフラグ
    ck_go_S = False        #   矢軸を縮めるかどうかのフラグ
    ck_loop = True         #   While文のループを継続するかどうかのフラグ
    while True:
        #
        #      イベントのチェック
        #
        for e in pygame.event.get():
            if e.type == pygame.QUIT:    #   Windowsのcloseボタンがクリックされたか
                pygame.quit()
                sys.exit()
            elif e.type == pygame.KEYDOWN:
                if e.key == pygame.K_RIGHT:   #   右矢印キーが押下されたか
                    ck_go_L = True
                if e.key == pygame.K_LEFT:    #   左矢印キーが押下されたか
                    ck_go_S = True
            elif e.type == pygame.KEYUP:      #   キーが離されたか
                ck_go_L = False
                ck_go_S = False
            elif e.type == pygame.MOUSEBUTTONDOWN:
                ms_point = pygame.mouse.get_pos()
                if ok_button.rect.collidepoint(ms_point):    #   ボタンがクリックされたか
                    screen.fill((255,255,255))               #   画面を白一色に消去
                    pygame.display.flip()
                    ck_loop = False
                    break
                
        if not ck_loop:    
            break              #    当該試行の終了

        if ck_go_L:
            disp_longer()      #     右矢軸を伸ばす
        if ck_go_S:
            disp_shorter()     #     右矢軸を縮める

    resps.append(350 + dL)     #     右矢軸の長さの記録
    pygame.time.wait(1000)     #     １秒待ち
            
            
pygame.quit()               

print('trials =\n', trials)
print('conds =\n', conds)
print('resps =\n', resps)

data = pd.DataFrame({'trial':trials, 'cond':conds, 'resp':resps})
fl_nm = input('Output File Name(*.xlsx) = ')
with pd.ExcelWriter(fl_nm) as ex_writer:
    data.to_excel(ex_writer, index=False)       #   データをExcelファイルに保存
print(fl_nm, 'was saved.')
    

