import pandas as pd
import numpy as np
import scipy.stats as ss
import scipy.optimize as so
import matplotlib.pyplot as plt

fl_nm = input('Input File(*.xlsx) = ')
with pd.ExcelFile(fl_nm) as f_data:      #   ファイル名の設定
    data_xlsx = pd.read_excel(f_data)    #   DataFrame型への読み込み
print(data_xlsx)                         #   書き出し

trials = data_xlsx['trial']
conds  = data_xlsx['cond']
resps  = data_xlsx['resp']

#
#       U条件のデータの取り出し
#
idx_U = conds == 'U'
trials_U = trials[idx_U]
conds_U = conds[idx_U]
resps_U = resps[idx_U]
#
#       L条件のデータの取り出し
#
idx_L = conds == 'L'
trials_L = trials[idx_L]
conds_L = conds[idx_L]
resps_L = resps[idx_L]
#
#       ヒストグラム
#
plt.hist(resps_U, label = 'U')
plt.hist(resps_L, label = 'L')
plt.plot([350], [0], c='c', marker = 'o', markersize=20, label = '350')
plt.legend()
plt.show()

#
#      累積比率:U条件
#
resps_U_s = np.sort(resps_U)
cum_U = []
for i in range(len(resps_U_s)):
    cum_U.append((i+1)/len(resps_U_s))

#
#       累積比率：L条件
#
resps_L_s = np.sort(resps_L)
cum_L = []
for i in range(len(resps_L_s)):
    cum_L.append((i+1)/len(resps_L_s))

plt.plot(resps_U_s, cum_U, label = 'U')
plt.plot(resps_L_s, cum_L,  label = 'L')
plt.title('Cumulative graph')
plt.legend()
plt.show()
#
#        2乗誤差の和を求めるクラス型
#
class ss_cum:
    def __init__(self, resps_U_s, cum_U, resps_L_s, cum_L):
        self.resps_U_s = resps_U_s
        self.cum_U = cum_U
        self.resps_L_s = resps_L_s
        self.cum_L = cum_L

    def f_ss(self, m, s, b):
        v_ss = 0.0
        for i in range(len(self.resps_U_s)):
            v_ss += (self.cum_U[i] - ss.norm.cdf(resps_U_s[i],
                                                 m + b, s)) ** 2
        for i in range(len(self.resps_L_s)):
            v_ss += (self.cum_L[i] - ss.norm.cdf(resps_L_s[i],
                                                 m - b, s)) ** 2
        return v_ss


func_ss = ss_cum(resps_U_s, cum_U, resps_L_s, cum_L)
#
#       最小2乗法のための目的関数
#
def opt_f(x):
    m = x[0]
    s = x[1]**2
    b = x[2]
    return func_ss.f_ss(m, s, b)

#
#      初期値
#
x0 = np.empty(3)
x0[0] = np.median(resps)
x0[1] = np.std(resps) ** 0.5
x0[2] = 0
results = so.minimize(opt_f, x0)
m = results.x[0]
s = results.x[1] ** 2
b = results.x[2]
print('\nm =', m, '  s =', s, '  b =', b)

#
#      求めた結果のグラフ表示：累積分布
#
r_min = np.min(resps)
r_max = np.max(resps)
r_range = np.arange(r_min, r_max, 0.001)
Model_cum_U = ss.norm.cdf(r_range, m-b, s)
Model_cum_L = ss.norm.cdf(r_range, m+b, s)
Model_cum_NoBias = ss.norm.cdf(r_range, m, s)

plt.plot(r_range, Model_cum_U, label = 'U: model')
plt.plot(r_range, Model_cum_L, label = 'L: model')
plt.plot(r_range, Model_cum_NoBias, label = 'b = 0')
plt.plot(resps_U_s, cum_U, label = 'U: data')
plt.plot(resps_L_s, cum_L, label = 'L: data')
plt.title(f'mu = {m:.2f}, sgm = {s:.2f}, b = {b:.2f}')
plt.legend()
plt.show()

#
#       求めた結果のグラフ表示：比率分布
#
dns_U = ss.norm.pdf(r_range, m+b, s)
dns_L = ss.norm.pdf(r_range, m-b, s)
plt.plot(r_range, dns_U, label = 'U: model')
plt.plot(r_range, dns_L, label = 'L: model')
plt.hist(resps_U, density = True, label = 'U: data')
plt.hist(resps_L, density = True, label = 'L: data')
plt.plot([350], [0], c='c', marker = 'o', markersize=20, label = '350')
plt.legend()
plt.show()

